﻿    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    using System.Windows.Shapes;
    using System.Windows.Controls;
    using System.Windows.Media;
    using System.Windows;

    namespace howto_wpf_canvas_controls
    {
        public static class Extensions
        {
            #region Support Methods

            // Set a shape's Fill, Stroke, StrokeThickness,
            // and StrokeDashArray properties.
            private static void PrepareGraphicProperties(this Shape shape,
                Brush fill, Brush stroke,
                double stroke_thickness, DoubleCollection stroke_dash_array)
            {
                shape.Fill = fill;
                shape.Stroke = stroke;
                shape.StrokeThickness = stroke_thickness;
                if (stroke_dash_array != null)
                {
                    shape.StrokeDashArray = stroke_dash_array;
                }
            }

            // Set a shape's Left, Top, Width, Height, Fill,
            // Stroke, StrokeThickness, and StrokeDashArray properties.
            // Return the shape.
            private static void PrepareShape(this Shape shape,
                double left, double top, double width, double height,
                Brush fill, Brush stroke,
                double stroke_thickness, DoubleCollection stroke_dash_array)
            {
                shape.SetBounds(left, top, width, height);
                shape.PrepareGraphicProperties(fill, stroke,
                    stroke_thickness, stroke_dash_array);
            }

            // Set a shape's Left, Top, Width, and Height. Return the shape.
            private static void SetBounds(this Shape shape,
                double left, double top, double width, double height)
            {
                Canvas.SetLeft(shape, left);
                Canvas.SetTop(shape, top);
                shape.Width = width;
                shape.Height = height;
            }
            private static void SetBounds(this Shape shape, Rect rect)
            {
                shape.SetBounds(rect.Left, rect.Top, rect.Width, rect.Height);
            }

            public static Point GetCenter(this Shape shape)
            {
                double cx = (Canvas.GetLeft(shape) + Canvas.GetRight(shape)) / 2.0;
                double cy = (Canvas.GetTop(shape) + Canvas.GetBottom(shape)) / 2.0;
                return new Point(cx, cy);
            }

            public static Rect GetBounds(this Shape shape)
            {
                return new Rect(
                    Canvas.GetLeft(shape), Canvas.GetTop(shape),
                    shape.Width, shape.Height);
            }

            // Position shape1 over shape2.
            public static void MatchBounds(this Shape shape1, Shape shape2)
            {
                shape1.SetBounds(shape2.GetBounds());
            }

            #endregion Support Methods

            #region Rectangle

            public static Rectangle DrawRectangle(this Canvas canvas,
                double left, double top, double width, double height,
                Brush fill, Brush stroke,
                double stroke_thickness, DoubleCollection stroke_dash_array)
            {
                Rectangle rectangle = new Rectangle();
                rectangle.PrepareShape(left, top, width, height,
                    fill, stroke, stroke_thickness, stroke_dash_array);
                canvas.Children.Add(rectangle);
                return rectangle;
            }
            public static Rectangle DrawRectangle(this Canvas canvas,
                double left, double top, double width, double height,
                Brush fill, Brush stroke,
                double stroke_thickness)
            {
                return canvas.DrawRectangle(left, top, width, height,
                    fill, stroke, stroke_thickness, null);
            }
            public static Rectangle DrawRectangle(this Canvas canvas,
                double left, double top, double width, double height,
                Brush fill, Brush stroke)
            {
                return canvas.DrawRectangle(left, top, width, height,
                    fill, stroke, 1, null);
            }

            #endregion Rectangle

            #region Ellipse

            public static Ellipse DrawEllipse(this Canvas canvas,
                double left, double top, double width, double height,
                Brush fill, Brush stroke,
                double stroke_thickness, DoubleCollection stroke_dash_array)
            {
                Ellipse ellipse = new Ellipse();
                ellipse.PrepareShape(left, top, width, height,
                    fill, stroke, stroke_thickness, stroke_dash_array);
                canvas.Children.Add(ellipse);
                return ellipse;
            }
            public static Ellipse DrawEllipse(this Canvas canvas,
                double left, double top, double width, double height,
                Brush fill, Brush stroke,
                double stroke_thickness)
            {
                return canvas.DrawEllipse(left, top, width, height,
                    fill, stroke, stroke_thickness, null);
            }
            public static Ellipse DrawEllipse(this Canvas canvas,
                double left, double top, double width, double height,
                Brush fill, Brush stroke)
            {
                return canvas.DrawEllipse(left, top, width, height,
                    fill, stroke, 1, null);
            }

            #endregion Ellipse

            #region Polygon

            public static Polygon DrawPolygon(this Canvas canvas,
                PointCollection points, Brush fill, Brush stroke,
                double stroke_thickness, DoubleCollection stroke_dash_array)
            {
                Polygon polygon = new Polygon();
                polygon.Points = points;
                polygon.PrepareGraphicProperties(fill, stroke,
                    stroke_thickness, stroke_dash_array);
                canvas.Children.Add(polygon);
                return polygon;
            }
            public static Polygon DrawPolygon(this Canvas canvas,
                PointCollection points, Brush fill, Brush stroke,
                double stroke_thickness)
            {
                return canvas.DrawPolygon(points, fill, stroke, stroke_thickness, null);
            }
            public static Polygon DrawPolygon(this Canvas canvas,
                PointCollection points, Brush fill, Brush stroke)
            {
                return canvas.DrawPolygon(points, fill, stroke, 1, null);
            }

            #endregion Polygon

            #region Polyline

            public static Polyline DrawPolyline(this Canvas canvas,
                PointCollection points, Brush fill, Brush stroke,
                double stroke_thickness, DoubleCollection stroke_dash_array)
            {
                Polyline polyline = new Polyline();
                polyline.Points = points;
                polyline.PrepareGraphicProperties(fill, stroke,
                    stroke_thickness, stroke_dash_array);
                canvas.Children.Add(polyline);
                return polyline;
            }
            public static Polyline DrawPolyline(this Canvas canvas,
                PointCollection points, Brush fill, Brush stroke,
                double stroke_thickness)
            {
                return canvas.DrawPolyline(points, fill, stroke, stroke_thickness, null);
            }
            public static Polyline DrawPolyline(this Canvas canvas,
                PointCollection points, Brush fill, Brush stroke)
            {
                return canvas.DrawPolyline(points, fill, stroke, 1, null);
            }

            #endregion Polyline

            #region Label

            public static Label DrawLabel(this Canvas canvas,
                double left, double top, double width, double height,
                string text, string font_family, double font_size,
                FontWeight font_weight, FontStyle font_style,
                Brush border_brush, Thickness border_thickness,
                Brush background_brush, Brush foreground_brush,
                HorizontalAlignment horizontal_alignment,
                VerticalAlignment vertical_alignment)
            {
                Label label = new Label();
                label.Content = text;
                label.FontFamily = new FontFamily(font_family);
                label.FontSize = font_size;
                label.FontWeight = font_weight;
                label.FontStyle = font_style;

                Canvas.SetLeft(label, left);
                Canvas.SetTop(label, top);
                label.Width = width;
                label.Height = height;

                label.BorderBrush = border_brush;
                label.BorderThickness = border_thickness;
                label.Background = background_brush;
                label.Foreground = foreground_brush;
                label.HorizontalContentAlignment = horizontal_alignment;
                label.VerticalContentAlignment = vertical_alignment;

                canvas.Children.Add(label);
                return label;
            }

            // Draw a label with a uniform border thickness.
            public static Label DrawLabel(this Canvas canvas,
                double left, double top, double width, double height,
                string text, string font_family, double font_size,
                FontWeight font_weight, FontStyle font_style,
                Brush border_brush, double border_thickness,
                Brush background_brush, Brush foreground_brush,
                HorizontalAlignment horizontal_alignment,
                VerticalAlignment vertical_alignment)
            {
                return canvas.DrawLabel(left, top, width, height,
                    text, font_family, font_size,
                    font_weight, font_style,
                    border_brush, new Thickness(border_thickness),
                    background_brush, foreground_brush,
                    horizontal_alignment, vertical_alignment);
            }

            // Draw a centered label.
            public static Label DrawLabel(this Canvas canvas,
                double left, double top, double width, double height,
                string text, string font_family, double font_size,
                FontWeight font_weight, FontStyle font_style,
                Brush border_brush, double border_thickness,
                Brush background_brush, Brush foreground_brush)
            {
                return canvas.DrawLabel(left, top, width, height,
                    text, font_family, font_size,
                    font_weight, font_style,
                    border_brush, new Thickness(border_thickness),
                    background_brush, foreground_brush, 
                    HorizontalAlignment.Center, VerticalAlignment.Center);
            }

            #endregion Label

            #region Line

            public static Line DrawLine(this Canvas canvas,
                double x1, double y1, double x2, double y2,
                Brush stroke, double stroke_thickness,
                DoubleCollection stroke_dash_array)
            {
                Line line = new Line();
                line.X1 = x1;
                line.Y1 = y1;
                line.X2 = x2;
                line.Y2 = y2;
                line.Stroke = stroke;
                line.StrokeThickness = stroke_thickness;
                if (stroke_dash_array != null)
                {
                    line.StrokeDashArray = stroke_dash_array;
                }
                canvas.Children.Add(line);
                return line;
            }
            public static Line DrawLine(this Canvas canvas,
                double x1, double y1, double x2, double y2,
                Brush stroke, double stroke_thickness)
            {
                return canvas.DrawLine(x1, y1, x2, y2,
                    stroke, stroke_thickness, null);
            }
            public static Line DrawLine(this Canvas canvas,
                double x1, double y1, double x2, double y2,
                Brush stroke)
            {
                return canvas.DrawLine(x1, y1, x2, y2,
                    stroke, 1, null);
            }

            #endregion Line

            #region Dot

            public static Ellipse DrawDot(this Canvas canvas,
                double cx, double cy, double radius,
                Brush fill, Brush stroke,
                double stroke_thickness, DoubleCollection stroke_dash_array)
            {
                return canvas.DrawEllipse(
                    cx - radius, cy - radius, 2.0 * radius, 2.0 * radius,
                    fill, stroke, stroke_thickness, stroke_dash_array);
            }

            public static Ellipse DrawDot(this Canvas canvas,
                out Label label,
                double cx, double cy, double radius,
                Brush fill, Brush stroke,
                double stroke_thickness, DoubleCollection stroke_dash_array,
                string text, string font_family, double font_size,
                FontWeight font_weight, FontStyle font_style,
                Brush border_brush, Thickness border_thickness,
                Brush background_brush, Brush foreground_brush)
            {
                Ellipse ellipse = canvas.DrawDot(
                    cx, cy, radius,
                    fill, stroke, stroke_thickness, stroke_dash_array);

                label = canvas.DrawLabel(
                    cx - radius, cy - radius, 2.0 * radius, 2.0 * radius,
                    text, font_family, font_size,
                    font_weight, font_style,
                    border_brush, border_thickness,
                    background_brush, foreground_brush,
                    HorizontalAlignment.Center, VerticalAlignment.Center);
                return ellipse;
            }
            public static Ellipse DrawDot(this Canvas canvas,
                out Label label,
                double cx, double cy, double radius,
                Brush fill, Brush stroke,
                double stroke_thickness, DoubleCollection stroke_dash_array,
                string text, string font_family, double font_size,
                FontWeight font_weight, FontStyle font_style,
                Brush background_brush, Brush foreground_brush)
            {
                return canvas.DrawDot(out label,
                    cx, cy, radius,
                    fill, stroke, stroke_thickness, stroke_dash_array,
                    text, font_family, font_size,
                    font_weight, font_style,
                    null, new Thickness(),
                    background_brush, foreground_brush);
            }

            #endregion Dot

        }
    }
